# Get content from the config.json
$config = Get-Content -Path .\config.json | ConvertFrom-Json

$xu_location = $config.xu_location
$xu_server = $config.xu_server
$xu_port = $config.xu_port
$sas_uri = $config.sas_uri
$end_of_upload_api = $config.end_of_upload_api

$extraction_start_date = $config.extraction_start_date
$extraction_end_date = $config.extraction_end_date
$language = $config.language
$exchange_rate_type = $config.exchange_rate_type

# Create a new log file for every run
$logFile = ".\extraction.log"

if (Test-Path -Path .\$logFile -PathType Leaf) {
    Remove-Item .\$logFile
}

# Function for appending text to the log file
function WriteLog
{
Param ([string]$logText)
$timestamp = (Get-Date).toString("dd/MM/yyyy HH:mm:ss")
Add-content $logFile -value "$timestamp $logText"
Write-Output $logText
}

# Function for checking the error code of extractions
function CheckExtractionError
{
Param ([string]$extraction)
if ($LASTEXITCODE -ne 0) {
    WriteLog("$extraction failed with return code: $LASTEXITCODE")
    Set-Variable -Name "extraction_failed" -Value 1 -Scope global
}
}

# Check whether the process-mining-data-uploader.exe can be found
if (-not(Test-Path -Path .\process-mining-data-uploader.exe -PathType Leaf)) {
    WriteLog("No process-mining-data-uploader.exe found.")
    Exit
}

# If at least one extraction fails, do not upload to blob storage.
Set-Variable -Name "extraction_failed" -Value 0 -Scope global

# Extraction commands
# Transactional tables
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_AFVC" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_AFVC')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_AFVV" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_AFVV')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_AFIH" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_AFIH')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_AFKO" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_AFKO')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_AUFK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_AUFK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_JCDS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_JCDS')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_QMEL" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_QMEL')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_CDHDR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_CDHDR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_CDPOS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('EM_CDPOS')

# Master data tables 
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_USR21"
CheckExtractionError('EM_USR21')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_USR02"
CheckExtractionError('EM_USR02')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_ADCP"
CheckExtractionError('EM_ADCP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_ADRP"
CheckExtractionError('EM_ADRP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_TKA01"
CheckExtractionError('EM_TKA01')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_T001W"
CheckExtractionError('EM_T001W')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_TCURX"
CheckExtractionError('EM_TCURX')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_T003P" -o "language=$language"
CheckExtractionError('EM_T003P')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_TJ02T" -o "language=$language"
CheckExtractionError('EM_TJ02T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_EQKT" -o "language=$language"
CheckExtractionError('EM_EQKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_TCURR" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('EM_TCURR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "EM_TCURF" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('EM_TCURF')

# Upload to the blob storage only if extraction is successful.
if ($extraction_failed) {
    Exit
} else {
    .\process-mining-data-uploader.exe "$xu_location\em_output" $sas_uri $end_of_upload_api | tee $logFile
}
